package org.jboss.security;

package .tmp;

import java.security.Principal;
import java.util.HashMap;
import javax.security.auth.Subject;
import org.jboss.logging.Logger;
import org.jboss.security.RunAsIdentity;
import org.jboss.security.SecurityActions;

public final class SecurityAssociation {
  private static Logger log = Logger.getLogger(org.jboss.security.SecurityAssociation.class);
  
  private static boolean trace;
  
  private static boolean server;
  
  private static Principal principal;
  
  private static Object credential;
  
  private static ThreadLocal threadPrincipal;
  
  private static ThreadLocal threadCredential;
  
  private static ThreadLocal threadContextMap;
  
  private static RunAsThreadLocalStack threadRunAsStacks;
  
  private static SubjectThreadLocalStack threadSubjectStacks;
  
  private static final RuntimePermission getPrincipalInfoPermission = new RuntimePermission("org.jboss.security.SecurityAssociation.getPrincipalInfo");
  
  private static final RuntimePermission getSubjectPermission = new RuntimePermission("org.jboss.security.SecurityAssociation.getSubject");
  
  private static final RuntimePermission setPrincipalInfoPermission = new RuntimePermission("org.jboss.security.SecurityAssociation.setPrincipalInfo");
  
  private static final RuntimePermission setServerPermission = new RuntimePermission("org.jboss.security.SecurityAssociation.setServer");
  
  private static final RuntimePermission setRunAsIdentity = new RuntimePermission("org.jboss.security.SecurityAssociation.setRunAsRole");
  
  private static final RuntimePermission getContextInfo = new RuntimePermission("org.jboss.security.SecurityAssociation.accessContextInfo", "get");
  
  private static final RuntimePermission setContextInfo = new RuntimePermission("org.jboss.security.SecurityAssociation.accessContextInfo", "set");
  
  static {
    String flag = SecurityActions.getProperty("org.jboss.security.SecurityAssociation.ThreadLocal", "false");
    boolean useThreadLocal = Boolean.valueOf(flag).booleanValue();
    log.debug("Using ThreadLocal: " + useThreadLocal);
    trace = log.isTraceEnabled();
    if (useThreadLocal) {
      threadPrincipal = new ThreadLocal();
      threadCredential = new ThreadLocal();
      threadContextMap = (ThreadLocal)new Object();
    } else {
      threadPrincipal = new InheritableThreadLocal();
      threadCredential = new InheritableThreadLocal();
      threadContextMap = (ThreadLocal)new HashMapInheritableLocal(null);
    } 
    threadRunAsStacks = new RunAsThreadLocalStack(useThreadLocal);
    threadSubjectStacks = new SubjectThreadLocalStack(useThreadLocal);
  }
  
  public static Principal getPrincipal() {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(getPrincipalInfoPermission); 
    Principal thePrincipal = principal;
    if (server)
      thePrincipal = threadPrincipal.get(); 
    if (trace)
      log.trace("getPrincipal, principal=" + thePrincipal); 
    return thePrincipal;
  }
  
  public static Principal getCallerPrincipal() {
    Principal principal;
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(getPrincipalInfoPermission); 
    RunAsIdentity runAsIdentity = peekRunAsIdentity(1);
    if (runAsIdentity == null)
      if (server) {
        principal = threadPrincipal.get();
      } else {
        principal = principal;
      }  
    if (trace)
      log.trace("getCallerPrincipal, principal=" + principal); 
    return principal;
  }
  
  public static Object getCredential() {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(getPrincipalInfoPermission); 
    if (server)
      return threadCredential.get(); 
    return credential;
  }
  
  public static Subject getSubject() {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(getSubjectPermission); 
    SubjectContext sc = threadSubjectStacks.peek();
    if (trace)
      log.trace("getSubject, sc=" + sc); 
    Subject subject = null;
    if (sc != null)
      subject = sc.getSubject(); 
    return subject;
  }
  
  public static void setPrincipal(Principal principal) {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(setPrincipalInfoPermission); 
    if (trace)
      log.trace("setPrincipal, p=" + principal + ", server=" + server); 
    if (server) {
      threadPrincipal.set(principal);
    } else {
      org.jboss.security.SecurityAssociation.principal = principal;
    } 
    SubjectContext sc = threadSubjectStacks.peek();
    if (sc == null) {
      sc = new SubjectContext();
      threadSubjectStacks.push(sc);
    } else if ((sc.getFlags() & 0x2) != 0) {
      sc = new SubjectContext();
      threadSubjectStacks.push(sc);
    } 
    sc.setPrincipal(principal);
    if (trace)
      log.trace("setPrincipal, sc=" + sc); 
  }
  
  public static void setCredential(Object credential) {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(setPrincipalInfoPermission); 
    if (server) {
      threadCredential.set(credential);
    } else {
      org.jboss.security.SecurityAssociation.credential = credential;
    } 
    SubjectContext sc = threadSubjectStacks.peek();
    if (sc == null) {
      sc = new SubjectContext();
      threadSubjectStacks.push(sc);
    } else if ((sc.getFlags() & 0x4) != 0) {
      sc = new SubjectContext();
      threadSubjectStacks.push(sc);
    } 
    sc.setCredential(credential);
    if (trace)
      log.trace("setCredential, sc=" + sc); 
  }
  
  public static void setSubject(Subject subject) {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(setPrincipalInfoPermission); 
    if (trace)
      log.trace("setSubject, s=" + subject + ", server=" + server); 
    SubjectContext sc = threadSubjectStacks.peek();
    if (sc == null) {
      sc = new SubjectContext();
      threadSubjectStacks.push(sc);
    } else if ((sc.getFlags() & 0x1) != 0) {
      sc = new SubjectContext();
      threadSubjectStacks.push(sc);
    } 
    sc.setSubject(subject);
    if (trace)
      log.trace("setSubject, sc=" + sc); 
  }
  
  public static Object getContextInfo(Object key) {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(getContextInfo); 
    HashMap contextInfo = threadContextMap.get();
    return contextInfo.get(key);
  }
  
  public static Object setContextInfo(Object key, Object value) {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(setContextInfo); 
    HashMap contextInfo = threadContextMap.get();
    return contextInfo.put(key, value);
  }
  
  public static void pushSubjectContext(Subject subject, Principal principal, Object credential) {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(setPrincipalInfoPermission); 
    if (server) {
      threadPrincipal.set(principal);
      threadCredential.set(credential);
    } else {
      org.jboss.security.SecurityAssociation.principal = principal;
      org.jboss.security.SecurityAssociation.credential = credential;
    } 
    SubjectContext sc = new SubjectContext(subject, principal, credential);
    threadSubjectStacks.push(sc);
    if (trace)
      log.trace("pushSubjectContext, subject=" + subject + ", sc=" + sc); 
  }
  
  public static void dupSubjectContext() {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(setPrincipalInfoPermission); 
    SubjectContext sc = threadSubjectStacks.dup();
    if (trace)
      log.trace("dupSubjectContext, sc=" + sc); 
  }
  
  public static SubjectContext popSubjectContext() {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(setPrincipalInfoPermission); 
    SubjectContext sc = threadSubjectStacks.pop();
    if (trace)
      log.trace("popSubjectContext, sc=" + sc); 
    return sc;
  }
  
  public static void clear() {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(setPrincipalInfoPermission); 
    if (trace)
      log.trace("clear, server=" + server); 
    if (server == true) {
      threadPrincipal.set(null);
      threadCredential.set(null);
    } else {
      principal = null;
      credential = null;
    } 
    threadSubjectStacks.clear();
  }
  
  public static void pushRunAsIdentity(RunAsIdentity runAs) {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(setRunAsIdentity); 
    if (trace)
      log.trace("pushRunAsIdentity, runAs=" + runAs); 
    threadRunAsStacks.push(runAs);
  }
  
  public static RunAsIdentity popRunAsIdentity() {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(setRunAsIdentity); 
    RunAsIdentity runAs = threadRunAsStacks.pop();
    if (trace)
      log.trace("popRunAsIdentity, runAs=" + runAs); 
    return runAs;
  }
  
  public static RunAsIdentity peekRunAsIdentity() {
    return peekRunAsIdentity(0);
  }
  
  public static RunAsIdentity peekRunAsIdentity(int depth) {
    RunAsIdentity runAs = threadRunAsStacks.peek(depth);
    return runAs;
  }
  
  public static void setServer() {
    SecurityManager sm = System.getSecurityManager();
    if (sm != null)
      sm.checkPermission(setServerPermission); 
    server = true;
  }
}
